#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time    : 2019/5/1 23:26
# @Email  : jtyoui@qq.com
# @Software: PyCharm
from jtyoui.error import DownLoadDataError
from jtyoui.decorator import deprecationWarning
from jtyoui.algorithm import dict_create_tree
from platform import platform
import time
import os
import bz2
import json
import re

re_id_card = """
(?P<province>[^省]+省|.+自治区)
(?P<city>[^自治州]+自治州|[^市]+市|[^盟]+盟|[^地区]+地区|.+区划)
(?P<county>[^市]+市|[^县]+县|[^旗]+旗|[^区]+区)?
(?P<town>[^区]+区|[^镇]+镇)?
(?P<village>.*)
""".replace('\n', '')

_Address = {}
_TREE = None


def _download_address_file(file_address_path):
    """下载地址数据包
    :param file_address_path:保存地址文件的路径
    """
    from urllib.request import urlretrieve
    url = 'https://dev.tencent.com/u/zhangwei0530/p/logo/git/raw/master/rear.bz2'
    place = urlretrieve(url, file_address_path)  # 下载
    print('---------验证数据-------')
    time.sleep(2)
    if not os.path.exists(file_address_path):
        print('下载失败')
        return False
    elif os.stat(file_address_path).st_size != 2292518:
        print('下载失败、移除无效文件！')
        os.remove(file_address_path)
        return False
    else:
        print('\033[1;33m' + place[0])
    return True


def load_address_file(file_address_path):
    """加载地址文件数据
    :param file_address_path: 加载地址文件的路径，没有地址文件默认自动下载。
    :return: 地址文件数据，类型字典
    """
    file_address_path = os.path.abspath(file_address_path) + os.sep + 'address'
    if not os.path.exists(file_address_path):
        if not _download_address_file(file_address_path):
            raise DownLoadDataError('检查网络设置，下载地址文件数据异常！')
    bz = bz2.BZ2File(file_address_path)
    text = bz.read().decode('utf-8')
    data = text[512:-1134]
    address = json.loads(data, encoding='utf8')
    return address


@deprecationWarning
def find_address(name):
    """查询地址。输入一个地名，查到这个名字的详细地址：比如输入：大连市、朝阳区、遵义县、卡比村等
    :param name: 输入一个地址。
    :return: 地址的信息
    """
    assert True if name else False, '输入的字符串不能为空'
    global _Address
    if not _Address:
        if 'Windows' in platform():
            path = r'D://'
        else:
            path = r'./'
        _Address = load_address_file(path)
    address = []
    for province in _Address:
        city_s = _Address[province]
        for city in city_s:
            districts = city_s[city]
            for district in districts:
                towns = districts[district]
                for town in towns:
                    villages = towns[town]
                    for village in villages:
                        if name in village:
                            addr = province + ' ' + city + ' ' + district + ' ' + town + ' ' + village
                            address.append(addr)
                    if name in town:
                        addr = province + ' ' + city + ' ' + district + ' ' + town
                        address.append(addr)
                if name in district:
                    addr = province + ' ' + city + ' ' + district
                    address.append(addr)
            if name in city:
                addr = province + ' ' + city
                address.append(addr)
        if name in province:
            addr = province
            address.append(addr)
    return address


def find_identity_card_address(card_addr):
    names = re.match(re_id_card, card_addr)
    province = names.group('province')
    city = names.group('city')
    county = names.group('county')
    town = names.group('town')
    village = names.group('village')
    return province, city, county, town, village


def finds_address(data, name: str):
    """查询地址。输入一个地名，查到这个名字的详细地址：比如输入：大连市、朝阳区、遵义县、卡比村等
    :param data: 地址数据
    :param name: 输入一个地址名
    :return: 地址的信息
    """
    global _TREE
    if not _TREE:
        _TREE = dict_create_tree(data)
    return _TREE.search_tree_value(name)


if __name__ == '__main__':
    import pprint

    pprint.pprint(find_address('晋安'))  # 已废弃，建议用：finds_address
    print(find_identity_card_address('贵州省贵阳市南明区花果园延安南路28号'))

    print('---------------------搜索树查找，第一次比较慢------------------------------------------')
    for t in finds_address(load_address_file('D://'), '晋安'):
        print(t)

    for t in finds_address(load_address_file(r'D://'), '遵义县'):
        print(t)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/10/28 15:27
# @Author: Jtyoui@qq.com
import collections


class Tree:
    def __init__(self, value=None, parent=None):
        self.value = value
        self.node = []
        self.parent = parent

    def add_child(self, node):
        self.node.append(node)

    def search_tree(self, value: str, ls):
        """多叉搜索树
        :param value: 树上一个值
        :param ls:树集合
        """
        if value in self.value:
            ls.append(self)
        else:
            for t in self.node:
                t.search_tree(value, ls)

    def search_tree_value(self, value: str):
        """搜索树的路径
        :param value:树上的一个值
        :return:树支的路径
        """
        ls = []
        self.search_tree(value, ls)
        for trees in ls:
            yield trees.node_parent_value()

    def node_parent_value(self):
        """知道一个节点，打印该节点的所有值（路径）
        :return: 返回该节点路径上的所有值
        """
        ls = []
        node = self
        while node.parent:
            ls.append(node.value)
            node = node.parent

        return '-'.join(reversed(ls))


def dict_create_tree(data: dict, tree: Tree = Tree(value='Root')):
    """创建下面的树结构
                a
            b   c   d
        e f g |g h| k m
    :param data: 创建树型结构，对照上面，例如：ds = {'a': {'b': ['e', 'f', 'g'], 'c': ['g', 'h'], 'd': ['k', 'm']}}
    :param tree: 默认为上一层树结构，不需要传入
    :return: 一颗自动带有root根目录的树结构
    """
    for d in data:
        t = Tree(value=d, parent=tree)
        tree.add_child(t)
        if isinstance(data, dict):
            dict_create_tree(data[d], t)
    return tree


def tree():
    """创建一颗简单树
    x=tree()
    x['a']['b']=1
    :return: 树
    """
    return collections.defaultdict(tree)


if __name__ == '__main__':
    """创建下面的树结构
            a
        b   c   d
    e f g |g h| k m
    """
    print('----------------------创建树-------------------------------')
    ds = {'a': {'b': ['e', 'f', 'g'], 'c': ['g', 'h'], 'd': ['k', 'm']}}
    ts = dict_create_tree(ds)
    print(ts)

    print('--------------------搜索树对象---------------------------------')
    tree_object = []
    ts.search_tree('g', tree_object)
    print(tree_object)

    print('---------------------打印路径------------------------------------')
    print(tree_object[0].node_parent_value())

    print('-----------------------搜索树的路径-------------------------')
    for i in ts.search_tree_value('g'):
        print(i)

    print('#############################################################')
    x = tree()
    x['a']['b'] = ['e', 'f', 'g']
    x['a']['c'] = ['g', 'h']
    x['a']['d'] = ['k', 'm']
    print(x)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/10/11 15:06
# @Author: Jtyoui@qq.com

Nations = """
汉族 满族 蒙古族 回族 藏族 维吾尔族 苗族 彝族 壮族 布依族 侗族 瑶族 
白族 土家族 哈尼族 哈萨克族 傣族 黎族 傈僳族 佤族 畲族 高山族 拉祜族 
水族 东乡族 纳西族 景颇族 柯尔克孜族 土族 达斡尔族 仫佬族 羌族 布朗族 
撒拉族 毛南族 仡佬族 锡伯族 阿昌族 普米族 朝鲜族 塔吉克族 怒族 乌孜别克族 
俄罗斯族 鄂温克族 德昂族 保安族 裕固族 京族 塔塔尔族 独龙族 鄂伦春族 
赫哲族 门巴族 珞巴族 基诺族
""".strip().split()

NationsOther = """
僜人 蔡家人 菜族人 老品人 八甲人 夏尔巴人 土克曼人 摩梭人 克木人 穿青人
""".strip().split()

if __name__ == '__main__':
    print(Nations)
    print(NationsOther)



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/8/23 13:39
# @Author: Jtyoui@qq.com
from functools import partial

import paddle.fluid as fluid
import paddle.fluid.layers as layers


def multi_head_attention(queries,
                         keys,
                         values,
                         attn_bias,
                         d_key,
                         d_value,
                         d_model,
                         n_head=1,
                         dropout_rate=0.,
                         cache=None,
                         param_initializer=None,
                         name='multi_head_att'):
    keys = queries if keys is None else keys
    values = keys if values is None else values

    if not (len(queries.shape) == len(keys.shape) == len(values.shape) == 3):
        raise ValueError(
            "Inputs: quries, keys and values should all be 3-D tensors.")

    def __compute_qkv(queries, keys, values, n_head, d_key, d_value):
        q = layers.fc(input=queries,
                      size=d_key * n_head,
                      num_flatten_dims=2,
                      param_attr=fluid.ParamAttr(
                          name=name + '_query_fc.w_0',
                          initializer=param_initializer),
                      bias_attr=name + '_query_fc.b_0')
        k = layers.fc(input=keys,
                      size=d_key * n_head,
                      num_flatten_dims=2,
                      param_attr=fluid.ParamAttr(
                          name=name + '_key_fc.w_0',
                          initializer=param_initializer),
                      bias_attr=name + '_key_fc.b_0')
        v = layers.fc(input=values,
                      size=d_value * n_head,
                      num_flatten_dims=2,
                      param_attr=fluid.ParamAttr(
                          name=name + '_value_fc.w_0',
                          initializer=param_initializer),
                      bias_attr=name + '_value_fc.b_0')
        return q, k, v

    def __split_heads(x, n_head):

        hidden_size = x.shape[-1]
        reshaped = layers.reshape(
            x=x, shape=[0, 0, n_head, hidden_size // n_head], inplace=True)

        return layers.transpose(x=reshaped, perm=[0, 2, 1, 3])

    def __combine_heads(x):
        if len(x.shape) == 3: return x
        if len(x.shape) != 4:
            raise ValueError("Input(x) should be a 4-D Tensor.")

        trans_x = layers.transpose(x, perm=[0, 2, 1, 3])
        return layers.reshape(
            x=trans_x,
            shape=[0, 0, trans_x.shape[2] * trans_x.shape[3]], inplace=True)

    def scaled_dot_product_attention(q, k, v, attn_bias, d_key, dropout_rate):
        scaled_q = layers.scale(x=q, scale=d_key ** -0.5)
        product = layers.matmul(x=scaled_q, y=k, transpose_y=True)
        if attn_bias:
            product += attn_bias
        weights = layers.softmax(product)
        if dropout_rate:
            weights = layers.dropout(
                weights,
                dropout_prob=dropout_rate,
                dropout_implementation="upscale_in_train",
                is_test=False)
        out = layers.matmul(weights, v)
        return out

    q, k, v = __compute_qkv(queries, keys, values, n_head, d_key, d_value)

    if cache is not None:
        k = cache["k"] = layers.concat(
            [layers.reshape(
                cache["k"], shape=[0, 0, d_model]), k], axis=1)
        v = cache["v"] = layers.concat(
            [layers.reshape(
                cache["v"], shape=[0, 0, d_model]), v], axis=1)

    q = __split_heads(q, n_head)
    k = __split_heads(k, n_head)
    v = __split_heads(v, n_head)

    ctx_multiheads = scaled_dot_product_attention(q, k, v, attn_bias, d_key,
                                                  dropout_rate)

    out = __combine_heads(ctx_multiheads)


    proj_out = layers.fc(input=out,
                         size=d_model,
                         num_flatten_dims=2,
                         param_attr=fluid.ParamAttr(
                             name=name + '_output_fc.w_0',
                             initializer=param_initializer),
                         bias_attr=name + '_output_fc.b_0')
    return proj_out


def positionwise_feed_forward(x,
                              d_inner_hid,
                              d_hid,
                              dropout_rate,
                              hidden_act,
                              param_initializer=None,
                              name='ffn'):

    hidden = layers.fc(input=x,
                       size=d_inner_hid,
                       num_flatten_dims=2,
                       act=hidden_act,
                       param_attr=fluid.ParamAttr(
                           name=name + '_fc_0.w_0',
                           initializer=param_initializer),
                       bias_attr=name + '_fc_0.b_0')
    if dropout_rate:
        hidden = layers.dropout(
            hidden,
            dropout_prob=dropout_rate,
            dropout_implementation="upscale_in_train",
            is_test=False)
    out = layers.fc(input=hidden,
                    size=d_hid,
                    num_flatten_dims=2,
                    param_attr=fluid.ParamAttr(
                        name=name + '_fc_1.w_0', initializer=param_initializer),
                    bias_attr=name + '_fc_1.b_0')
    return out


def pre_post_process_layer(prev_out, out, process_cmd, dropout_rate=0., name=''):
    for cmd in process_cmd:
        if cmd == "a":  # add residual connection
            out = out + prev_out if prev_out else out
        elif cmd == "n":  # add layer normalization
            out_dtype = out.dtype
            if out_dtype == fluid.core.VarDesc.VarType.FP16:
                out = layers.cast(x=out, dtype="float32")
            out = layers.layer_norm(
                out,
                begin_norm_axis=len(out.shape) - 1,
                param_attr=fluid.ParamAttr(
                    name=name + '_layer_norm_scale',
                    initializer=fluid.initializer.Constant(1.)),
                bias_attr=fluid.ParamAttr(
                    name=name + '_layer_norm_bias',
                    initializer=fluid.initializer.Constant(0.)))
            if out_dtype == fluid.core.VarDesc.VarType.FP16:
                out = layers.cast(x=out, dtype="float16")
        elif cmd == "d":  # add dropout
            if dropout_rate:
                out = layers.dropout(
                    out,
                    dropout_prob=dropout_rate,
                    dropout_implementation="upscale_in_train",
                    is_test=False)
    return out


pre_process_layer = partial(pre_post_process_layer, None)
post_process_layer = pre_post_process_layer


def encoder_layer(enc_input,
                  attn_bias,
                  n_head,
                  d_key,
                  d_value,
                  d_model,
                  d_inner_hid,
                  prepostprocess_dropout,
                  attention_dropout,
                  relu_dropout,
                  hidden_act,
                  preprocess_cmd="n",
                  postprocess_cmd="da",
                  param_initializer=None,
                  name=''):
    attn_output = multi_head_attention(
        pre_process_layer(
            enc_input,
            preprocess_cmd,
            prepostprocess_dropout,
            name=name + '_pre_att'),
        None,
        None,
        attn_bias,
        d_key,
        d_value,
        d_model,
        n_head,
        attention_dropout,
        param_initializer=param_initializer,
        name=name + '_multi_head_att')
    attn_output = post_process_layer(
        enc_input,
        attn_output,
        postprocess_cmd,
        prepostprocess_dropout,
        name=name + '_post_att')
    ffd_output = positionwise_feed_forward(
        pre_process_layer(
            attn_output,
            preprocess_cmd,
            prepostprocess_dropout,
            name=name + '_pre_ffn'),
        d_inner_hid,
        d_model,
        relu_dropout,
        hidden_act,
        param_initializer=param_initializer,
        name=name + '_ffn')
    return post_process_layer(
        attn_output,
        ffd_output,
        postprocess_cmd,
        prepostprocess_dropout,
        name=name + '_post_ffn')


def encoder(n_layer, **kwargs):
    name = kwargs['name']
    encoder_output = None
    for i in range(n_layer):
        kwargs['name'] = name + '_layer_' + str(i)
        encoder_output = encoder_layer(**kwargs)
        kwargs['enc_input'] = encoder_output

    enc_output = pre_process_layer(encoder_output, kwargs['preprocess_cmd'], kwargs['prepostprocess_dropout'],
                                   name="post_encoder")

    return enc_output



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/11 11:43
# @Author: Jtyoui@qq.com
from jtyoui.decorator.warn import *
import re
from jtyoui.error import CoordinateLengthNotEqualError, ParameterNotEmptyError

"""
装饰器模式
"""


@warns('也废除，请使用functools.wraps函数来替代', DeprecationWarning)
def warps(func):
    """这个装饰器等效：functools.wraps(fun)"""

    def call(*args, **kwargs):
        return func(*args, **kwargs)

    call.__doc__ = func.__doc__
    call.__name__ = func.__name__
    call.__dict__.update(func.__dict__)
    return call


def replace_regular(re_, replace_):
    """
    根据正则来修改参数
    :param re_: 匹配的正则
    :param replace_: 替换正则的数据
    :return: 被替换完毕的参数
    """
    r = re.compile(re_)

    def remove_replace(fun):

        @functools.wraps(fun)
        def wraps(*args, **kwargs):
            args_, kwargs_ = list(args), {}
            for i in range(len(args)):
                if isinstance(args[i], str):
                    args_[i] = r.sub(replace_, args[i])
                else:
                    args_[i] = args[i]
            for k, v in kwargs.items():
                if isinstance(v, str):
                    kwargs_.setdefault(k, r.sub(replace_, v))
                else:
                    kwargs_.setdefault(k, v)

            return fun(*args_, **kwargs_)

        return wraps

    return remove_replace


def parameter_set_length(fun):
    """参数集合长度验证修饰器"""

    @functools.wraps(fun)
    def wraps(x, y):
        if not (x and y):
            raise ParameterNotEmptyError("参数不能为空")
        if len(x) != len(y):
            raise CoordinateLengthNotEqualError("两个参数长度不一致")
        return fun(x, y)

    return wraps


def singleton(cls, *args, **kwargs):
    """单列模式修饰器"""
    instances = {}

    @functools.wraps(cls, *args, **kwargs)
    def _singleton():
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    return _singleton


def coroutine(func):
    """自动激活协程装饰器"""

    @functools.wraps(func)
    def warp(*args, **kwargs):
        g = func(*args, **kwargs)
        next(g)
        return g

    return warp


if __name__ == '__main__':
    from jtyoui.regular import Non_Chinese


    @replace_regular(' ', '')
    def remove_blank(a, b):
        print(a, b)


    @replace_regular(Non_Chinese, '')
    def remove_non_chinese(a, b):
        print(a, b)


    remove_blank('你好  吗?', b='我  很好!')
    remove_non_chinese('你好#$%76#%吗wore?', b='我$%787word很好!')


    @singleton
    class A:
        pass


    a = A()
    b = A()
    print(id(a) == id(b))  # True


    @coroutine
    def receiver():
        n = 0
        while True:
            n = yield n + n


    print(receiver().send(10))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/7/28 11:58
# @Author: jtyoui@qq.com
import math
from jtyoui.tools import Tool


class TFIDF:

    def __init__(self):
        """增加IFIDF"""
        self.word = None
        self.length = 0
        self.data = None
        self.ls = None

    def input_str(self, str_):
        self.data = str_
        if not self.ls:
            self.input_ls(Tool(str_).split('[。？！.?!]', retain=False))

    def input_ls(self, ls):
        self.length = len(ls)
        self.ls = ls
        if not self.data:
            self.input_str('。'.join(ls))

    def get_tf_idf(self, word):
        """传入一个词语，获得重要性"""
        self.word = word
        c, t = 0, 0
        for line in self.ls:
            num = line.count(self.word)
            if num:
                t += num / len(line)
                c += 1
        return t * math.log10(self.length / c)


if __name__ == '__main__':
    tfidf = TFIDF()
    tfidf.input_str("这是只猫,猫是朋友。这是只狗,狗是朋友")
    print(tfidf.get_tf_idf('猫'))
    print(tfidf.get_tf_idf('这'))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/4/24 17:30
# @Author: Jtyoui@qq.com

from .tool import Tool  # 常见处理字符串的工具
from .times import StringTime  # 讲口语化的时间转为标准时间
from .SentenceMake import Sentence  # 增加造句
from .captcha import Captcha  # 验证码生成器
from ._pip import pips  # 安装模块
from .timestr import *  # 时间操作



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/18 9:33
# @Author: Jtyoui@qq.com
from jtyoui import parameter_set_length

"""
曼哈顿距离
顾名思义，在曼哈顿街区要从一个十字路口开车到另一个十字路口
，驾驶距离显然不是两点间的直线距离。这个实际驾驶距离就是“曼哈顿距离”。
曼哈顿距离也称为“城市街区距离”(City Block distance)。
"""


@parameter_set_length
def manhattan_distance(coordinate_p, coordinate_q):
    """
    曼哈顿距离
    :param coordinate_p: p坐标
    :param coordinate_q: q坐标
    :return: 曼哈顿距离值
    """
    point = (abs(x - y) for x, y in zip(coordinate_q, coordinate_p))
    return sum(point)


if __name__ == '__main__':
    print(manhattan_distance((1, 1), (7, 7)))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/8 13:31
# @Author: Jtyoui@qq.com
from jtyoui.ml import sigmoid, get_cost, TRAIN_DATA, TEST_LABEL
from random import normalvariate
import numpy as np

__description__ = """
FM(因子分解机)算法
"""


def initialize_v(n: int, k: int):
    """初始化交叉项
    :param n:特征个数
    :param k:FM模型的度
    :return:交叉项的系数权重
    """
    v = np.mat(np.zeros(shape=(n, k)))
    for i in range(n):
        for j in range(k):
            v[i, j] = normalvariate(0, 0.2)
    return v


def get_prediction(data, w0, w, v):
    """预测值
    :param data:特征
    :param w0:一次项权重
    :param w:常数项权重
    :param v:交叉项权重
    :return:预测结果
    """
    m = np.shape(data)[0]
    result = []
    for x in range(m):
        inter_1 = data[x] * v
        inter_2 = np.multiply(data[x], data[x]) * np.multiply(v, v)
        inter = np.sum(np.multiply(inter_1, inter_1) - inter_2) / 2.
        p = w0 + data[x] * w + inter
        pre = sigmoid(p[0, 0])
        result.append(pre)
    return result


def stop_grad_ascent(data: np.mat, label: np.mat, k: int, max_iter: int, alpha: float) -> (float, np.mat, np.mat):
    """利用随机梯度下降法训练FM模型
    :param data: 数据特征
    :param label:标签
    :param k:v的维度
    :param max_iter:最大迭代次数
    :param alpha:学习率
    :return:w0,w,v权重
    """
    m, n = np.shape(data)
    w = np.random.randn(n).reshape((n, 1))
    w0 = 0
    v = initialize_v(n, k)
    for it in range(max_iter):
        for x in range(m):
            inter_1 = data[x] * v
            inter_2 = np.multiply(data[x], data[x]) * np.multiply(v, v)
            inter = np.sum(np.multiply(inter_1, inter_1) - inter_2) / 2.
            p = w0 + data[x] * w + inter
            loss = sigmoid(label[x] * p[0, 0]) - 1
            w0 -= alpha * loss * label[x]
            for i in range(n):
                if data[x, i] != 0:
                    w[i, 0] -= alpha * loss * label[x] * data[x, i]
                for j in range(k):
                    v[i, j] -= alpha * loss * label[x] * (
                            data[x, i] * inter_1[0, j] - v[i, j] * data[x, i] * data[x, i])
        if it % 100 == 0:
            pre = get_prediction(np.mat(data), w0, w, v)
            print(get_cost(np.mat(pre), label))
    return w0, w, v


if __name__ == '__main__':
    weight = stop_grad_ascent(TRAIN_DATA, TEST_LABEL, 3, 1000, 0.1)
    print(get_prediction(np.mat([[1, 1 / 24, 10 / 60, 32 / 60]]), *weight))



#!/usr/bin/python3.7
# -*-coding:utf-8-*-
# @Time:2019/6/2115:40
# @Author:Jtyoui@qq.com
from jtyoui.data.wordNature import word_nature
import re

_wn = word_nature()


def continuous_pause(sentence: str) -> str:
    """去除多重断句符号"""
    return re.sub('[，。,.]{2,}', '，', sentence)


def punctuate(sentence):
    """根据词性进行简单校验断句"""
    p = continuous_pause(sentence)
    sent = ''
    max_len = len(p)
    for index, word in enumerate(p):
        if word in '，。,.':
            if index == 0:
                continue
            elif p[index - 1] in _wn['aw_tone']:
                continue
            elif max_len == index + 1:
                if p[index] in _wn['aw_structure']:
                    continue
            elif p[index + 1] in _wn['aw_structure']:
                continue
        sent += word
    return sent


if __name__ == '__main__':
    print(punctuate('你在哪.,里。.，我在天安门。.，'))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/9/29 15:39
# @Author: Jtyoui@qq.com
import warnings
import functools


def warns(message, category=None):
    """警告装饰器
    :param message:警告信息
    :param category: 警告类型：默认是None
    :return: 装饰函数的对象
    """

    def _(func):
        @functools.wraps(func)
        def warp(*args, **kwargs):
            warnings.warn(message, category, stacklevel=2)
            return func(*args, **kwargs)

        return warp

    return _


# 废除警告
deprecationWarning = warns('该函数已废弃！', DeprecationWarning)

# 运行警告
runtimeWarning = warns('该函数运行期间可能存在问题！', RuntimeWarning)

if __name__ == '__main__':
    @runtimeWarning
    def w(): ...


    w()



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/18 17:16
# @Author: Jtyoui@qq.com

"""
闵可夫斯基距离
闵氏空间指狭义相对论中由一个时间维和三个空间维组成的时空，
为俄裔德国数学家闵可夫斯基(1864-1909)最先表述。
他的平坦空间的概念以及表示为特殊距离量的几何学是与狭义相对论的要求相一致的。
闵可夫斯基空间不同于牛顿力学的平坦空间
当dimension=1时，得到绝对值距离，也叫曼哈顿距离
当dimension=2时，得到欧几里德距离
令dimension=无穷大(math.inf)，得到切比雪夫距离
"""


def minkowski_distance(coordinate_p, coordinate_q, dimension):
    """
    闵可夫斯基距离
    :param coordinate_p: p坐标
    :param coordinate_q: q坐标
    :param dimension:闵可夫斯基维度
    :return: 闵可夫斯基距离
    """
    all_ = []
    for x, y in zip(coordinate_p, coordinate_q):
        p = abs(x - y) ** dimension
        all_.append(p)
    return pow(sum(all_), 1 / dimension)


if __name__ == '__main__':
    print(minkowski_distance([3, 0], [0, 4], 3))



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/14 17:59
# @Author: Jtyoui

Municipality_Directly_Under_The_Central_Government = ['北京市', '上海市', '天津市', '重庆市']  # 直辖市

# 安徽
AnHui = ['合肥', '安庆', '蚌埠', '亳州', '巢湖', '池州', '滁州', '阜阳', '淮北', '淮南', '黄山', '六安', '马鞍山', '宿州', '铜陵', '芜湖', '宣城']

# 福建
FuJian = ['福州', '龙岩', '南平', '宁德', '莆田', '泉州', '三明', '厦门', '漳州']

# 甘肃
GanSu = ['兰州', '白银', '定西', '甘南州', '嘉峪关', '金昌', '酒泉', '临夏州', '平凉', '庆阳', '天水', '武威', '张掖']

# 广东
GuangDong = ['广州', '潮州', '东莞', '佛山', '河源', '惠州', '江门', '揭阳', '茂名', '梅州', '清远', '汕头', '汕尾', '韶关', '深圳', '阳江', '云浮', '湛江',
             '肇庆', '中山', '珠海']

# 广西
GuangXi = ['南宁', '百色', '北海', '崇左', '防城港', '桂林', '贵港', '河池', '贺州', '来宾', '柳州', '钦州', '梧州', '玉林']

# 贵州
GuiZhou = ['贵阳', '安顺', '毕节地区', '六盘水', '铜仁地区', '遵义', '黔西南州']

# 海南
HaiNan = ['海口', '白沙', '保亭', '昌江', '儋州', '澄迈', '东方', '定安', '琼海', '琼中', '乐东', '临高', '陵水', '三亚', '屯昌', '万宁', '文昌', '五指山']

# 河北
HeBei = ['石家庄', '保定', '沧州', '承德', '邯郸', '衡水', '廊坊', '秦皇岛', '唐山', '邢台', '张家口']

# 河南
HeNan = ['郑州', '安阳', '鹤壁', '焦作', '开封', '洛阳', '漯河', '南阳', '平顶山', '濮阳', '三门峡', '商丘', '新乡', '信阳', '许昌', '周口', '驻马店']

# 黑龙江
HeiLongJiang = ['哈尔滨', '大庆', '大兴安岭地区', '鹤岗', '黑河', '鸡西', '佳木斯', '牡丹江', '七台河', '齐齐哈尔', '双鸭山', '绥化', '伊春']

# 湖北
HuBei = ['武汉', '鄂州', '恩施', '黄冈', '黄石', '荆门', '荆州', '潜江', '神农架林区', '十堰', '随州', '天门', '仙桃', '咸宁', '襄樊', '孝感', '宜昌']

# 湖南
HuNan = ['长沙', '常德', '郴州', '衡阳', '怀化', '娄底', '邵阳', '湘潭', '益阳', '永州', '岳阳', '张家界', '株洲']

# 江苏
JiangSu = ['南京', '常州', '淮安', '连云港', '南通', '苏州', '宿迁', '泰州', '无锡', '徐州', '盐城', '扬州', '镇江']

# 江西
JiangXi = ['南昌', '抚州', '赣州', '吉安', '景德镇', '九江', '萍乡', '上饶', '新余', '宜春', '鹰潭']

# 吉林
JiLin = ['长春', '白城', '白山', '吉林市', '辽源', '四平', '松原', '通化', '延边']

# 辽宁
LiaoNing = ['沈阳', '鞍山', '本溪', '朝阳', '大连', '丹东', '抚顺', '阜新', '葫芦岛', '锦州', '辽阳', '盘锦', '铁岭', '营口']

# 内蒙古
Inner_Mongolia = ['呼和浩特', '包头', '巴彦淖尔', '赤峰', '鄂尔多斯', '呼伦贝尔', '通辽', '乌海', '乌兰察布', '兴安盟']

# 宁夏
NingXia = ['银川', '固原', '石嘴山', '吴忠', '中卫']

# 青海
QingHai = ['西宁', '果洛州', '海东地区', '海北州', '海南州', '海西州', '黄南州', '玉树州']

# 山东
ShanDong = ['济南', '滨州', '东营', '德州', '菏泽', '济宁', '莱芜', '聊城', '临沂', '青岛', '日照', '泰安', '威海', '潍坊', '烟台', '枣庄', '淄博']

# 山西
ShanXi = ['太原', '长治', '大同', '晋城', '晋中', '临汾', '吕梁', '朔州', '忻州', '阳泉', '运城']

# 陕西
ShaAnXi = ['西安', '安康', '宝鸡', '汉中', '商洛', '铜川', '渭南', '咸阳', '延安', '榆林']

# 四川
SiChuan = ['成都', '阿坝州', '巴中', '达州', '德阳', '甘孜州', '广安', '广元', '乐山', '凉山州', '泸州', '南充', '眉山', '绵阳', '内江', '攀枝花', '遂宁',
           '雅安', '宜宾', '资阳', '自贡']

# 西藏
TiBet = ['拉萨', '阿里地区', '昌都地区', '林芝地区', '那曲地区', '日喀则地区', '山南地区']

# 新疆
XinJiang = ['乌鲁木齐', '阿拉尔', '阿克苏地区', '阿勒泰地区', '昌吉州', '哈密地区', '和田地区', '喀什地区', '克拉玛依', '石河子', '塔城地区', '吐鲁番地区']

# 云南
YunNan = ['昆明', '保山', '楚雄州', '大理州', '德宏州', '迪庆州', '红河州', '丽江', '临沧', '怒江州', '普洱', '曲靖', '昭通', '文山', '西双版纳', '玉溪']

# 浙江
ZheJiang = ['杭州', '湖州', '嘉兴', '金华', '丽水', '宁波', '衢州', '绍兴', '台州', '温州', '舟山']

Special_Administrative_Region = ['香港', '澳门', '台湾']  # 三个特别行政区

# 省
Province = ['安徽省', '福建省', '甘肃省', '广东省', '广西壮族自治区', '贵州省', '海南省', '河北省', '河南省', '黑龙江省',
            '台湾省', '内蒙古自治区', '宁夏回族自治区', '青海省', '山东省', '山西省', '陕西省', '四川省', '西藏自治区',
            '浙江省', '辽宁省' '江苏省', '江西省', '新疆维吾尔自治区', '云南省', '湖北省', '湖南省']

# 全国
Whole_Country = [AnHui, FuJian, GanSu, GuangDong, GuangXi, GuiZhou, HaiNan, HeBei, HeNan, HeiLongJiang, HuBei, HuNan,
                 JiangSu, JiangXi, JiLin, LiaoNing, Inner_Mongolia, NingXia, QingHai, ShanDong, ShanXi, ShaAnXi,
                 SiChuan, TiBet, XinJiang, YunNan, ZheJiang]



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time : 2019/2/19 0019
# @Email : jtyoui@qq.com
import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication


def send_qq_mail(from_addr, password, to_addr, content, subject='', files=None, host=('smtp.qq.com', 465)):
    """
    这个一个邮箱发送函数.默认是qq邮箱
    :param from_addr: 发送方邮箱
    :param password: 填入发送方邮箱的授权码
    :param to_addr: 收件人为多个收件人,通过;为间隔的字符串,比如: xx@qq.com;yy@qq.com
    :param content: 正文
    :param subject: 主题
    :param files: 附加
    :param host: 邮件传输协议
    :return: bool类型.打印成功和失败
    """
    text = MIMEText(content, _charset='utf-8')
    m = MIMEMultipart()
    if files:
        import os
        from email import encoders
        file_name = os.path.basename(files)  # 获得文件名字
        file = MIMEApplication(open(files, 'rb').read())
        file.add_header('Content-Disposition', 'attachment', filename=('GBK', '', file_name))
        encoders.encode_base64(file)  # 解决文件名乱码问题
        m.attach(file)
    m['Subject'] = subject
    m['From'] = from_addr
    m['To'] = to_addr
    m.attach(text)
    server = None
    try:
        server = smtplib.SMTP_SSL(*host)  # 安全模式
        server.login(from_addr, password)  # 登陆
        server.sendmail(from_addr, to_addr, m.as_string())  # 发送
        return True
    except smtplib.SMTPException as e:
        print(e)
        return False
    finally:
        if not server:
            server.quit()


# jfciwswgxlmedjei9

if __name__ == '__main__':
    flag = send_qq_mail('ptyoui@jtyoui.com', '授权码', 'jtyoui@jtyoui.com', '测试邮箱', '政治文件', files='政治.txt')
    if flag:
        print('发送成功!')
    else:
        print('发送失败!')



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/28 14:56
# @Author: Jtyoui@qq.com
from .flaskerror import flask_register_errors, flask_abort, flask_error_abort  # flask框架异常
from .request import content_type  # flask框架request的补充



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/6/6 10:24
# @Author: Jtyoui@qq.com

"""统计指标：混淆矩阵
True Positive(真正，TP)：将正类预测为正类数
True Negative(真负，TN)：将负类预测为负类数
False Positive(假正，FP)：将负类预测为正类数
False Negative(假负，FN)：将正类预测为负类数
P=TP+FP 预测对的
N=FN+TN 预测错的
PP=TP+FN 实际上是对的
NN=FP+TN 实际上是错误的
ALL=P+N=PP+NN=所有
"""


def confusion_matrix(simple, pred):
    """混淆矩阵
    :param simple:实际样本分类列表
    :param pred:预测样本分类列表
    :return TP,TN,FP,FN
    """
    tp, tn, fp, fn = 0, 0, 0, 0
    for s, p in zip(simple, pred):
        if s == p == 1:  # 将正类预测为正类数
            tp += 1
        elif s == p == 0:  # 将负类预测为负类数
            tn += 1
        elif s == 1 and p == 0:  # 将正类预测为负类数
            fn += 1
        else:  # 将负类预测为正类数
            fp += 1
    return tp, tn, fp, fn


def precision(tp, tn, fp, fn):
    """精确率=(TP+TN)/(TP+TN+FP+FN)"""
    return (tp + tn) / (tp + tn + fp + fn)


def recall(tp, tn, fp, fn):
    """召回率=TP/(TP+FN)"""
    return tp / (tp + fn)


def f_measure(tp, tn, fp, fn):
    """综合评价指标=(2*P*R)/(P+R)"""
    p = precision(tp, tn, fp, fn)
    r = recall(tp, tn, fp, fn)
    return (2 * p * r) / (p + r)


if __name__ == '__main__':
    simples = [1, 1, 0, 0, 0, 0, 1, 0, 0, 1]  # 实际分类的值
    prediction = [0, 1, 0, 1, 1, 0, 1, 1, 0, 1]  # 预测分类的值
    m = confusion_matrix(simple=simples, pred=prediction)  # 混淆矩阵
    p_ = precision(*m)  # 准确率
    r_ = recall(*m)  # 召回率
    f_ = f_measure(*m)  # f值
    print(p_, r_, f_)  # 0.6 0.75 0.6666666666666665



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/3/26 9:47
# @Author: Jtyoui@qq.com

from .neologism import Neologism  # 没有多线程
from .ThreadNeologism import thread_analysis  # 有多线程
from .NAA import NAA  # 自动过滤的算法
from .TS import TextSummary  # 精简摘要算法
from .tfidf import TFIDF  # 增加TFIDF
from ._pt import *  # 根据词语进行重新断句



#!/usr/bin/python3.7
# -*- coding: utf-8 -*-
# @Time  : 2019/11/28 10:51
# @Author: Jtyoui@qq.com
from jtyoui.data.constant import http_status_code


def flask_error_abort(code, jsonify, error, message=None, **kwargs):
    """手动抛出flask异常、并且增加error详细异常"""
    return flask_abort(code=code, jsonify=jsonify, message=message, error=error, **kwargs)


def flask_abort(code, jsonify, message=None, **kwargs):
    """手动抛出flask异常
    :param code: 状态码
    :param jsonify: 传入flask里面的jsonify对象。from flask import jsonify
    :param message: 返回的msg异常信息
    :param kwargs:其他jsonify参数
    :return:异常状态信息
    """
    if message is None:
        message = http_status_code.get(str(code), '')
    response = jsonify(msg=message, code=code, **kwargs)
    return response, code


def flask_register_errors(app, jsonify):
    """app启动处注册一个错误处理器
        from flask import jsonify,Flask
        app = Flask(__name__)
        register_errors(app, jsonify) #注册异常解析器
    :param app: app对象
    :param jsonify: jsonify对象
    """

    @app.errorhandler(400)
    def bad_request(e):
        return flask_abort(400, jsonify)

    @app.errorhandler(401)
    def verify_id(e):
        return flask_abort(401, jsonify)

    @app.errorhandler(403)
    def forbidden(e):
        return flask_abort(403, jsonify)

    @app.errorhandler(404)
    def database_not_found_error_handler(e):
        return flask_abort(404, jsonify)

    @app.errorhandler(405)
    def method_not_allowed(e):
        return flask_abort(405, jsonify)

    @app.errorhandler(500)
    def internal_server_error(e):
        return flask_abort(500, jsonify)

    @app.errorhandler(Exception)
    def default_error_handler(e):
        message = f'发生未处理的异常 -> {e}'
        return flask_abort(0, jsonify, message=message)


if __name__ == '__main__':
    from flask import Flask, jsonify, abort

    app = Flask(__name__)
    flask_register_errors(app, jsonify)  # 注册错误异常处理器


    @app.route('/hi', methods=['POST'])
    def hello():
        try:
            value = 1 / 0
            return jsonify(msg='请求成功！', data=value, code=200)  # 正常信息
        except ValueError:
            return flask_abort(600, jsonify, message='被除数不能为0')  # 有自定义的处理器来处理
        except KeyError:
            code = 400
        except Exception:  # 其他错误
            code = 500
        return abort(code)  # 由错误异常处理器来处理
